"""Module of Sonar Scanner commands"""

import sys
from enum import Enum
from pathlib import Path

from colorama import Fore

sys.path.append("./senior-ci")

from . import (
    ExitCode,
    exec_command,
    exec_command_and_return,
    exit_message,
    get_env_variable,
    print_message,
)
from .extensions import Extensions
from .graphql_client import GraphqlClient


class SonarScannerType(Enum):
    FLUTTER = "FLUTTER"
    MAVEN = "MAVEN"
    SDL = "SDL"


class SonarHelper:
    def __init__(
        self,
        extensions: Extensions = Extensions(),
        sonar_scanner_type: SonarScannerType = None,
    ) -> None:
        self.extensions = extensions
        self.sonar_scanner_type = sonar_scanner_type
        self.graphql_client = GraphqlClient()

        self.ci_project_path_slug = get_env_variable("CI_PROJECT_PATH_SLUG")
        self.ci_project_name = get_env_variable("CI_PROJECT_NAME")
        self.ci_commit_branch = get_env_variable("CI_COMMIT_BRANCH")

        self.sonar_url = get_env_variable("SONAR_HOST_URL")

    def scanner_analyse(self):
        self.extensions.before_sonar()

        print(" ")
        print_message(
            "Iniciando a execução do Sonar...",
            Fore.YELLOW,
        )
        print(" ")

        base_sonar_scanner = self._create_base_sonar_scanner()

        command_result_sonar = exec_command_and_return(base_sonar_scanner)

        print(command_result_sonar.output)

        if command_result_sonar.exit_code == ExitCode.ERROR:
            exit_message(
                f"Analise do sonarqube finalizada com erro, verifique no dashboard quais são os motivos e como ajusta-los: {self.sonar_url}/dashboard?branch={self.ci_commit_branch}&id={self.ci_project_path_slug}"
            )

        self._config()

        print(" ")
        print_message(
            "Analise estática feita pelo sonar finalizada com sucesso.",
            Fore.GREEN,
        )
        print(" ")

        self.extensions.after_sonar()

    def _create_base_sonar_scanner(self):
        ci_project_url = get_env_variable("CI_PROJECT_URL")

        is_shallow = exec_command_and_return("git rev-parse --is-shallow-repository")

        if is_shallow.output.rstrip() == "true":
            exec_command("git fetch --unshallow")

        exec_command("git fetch --all")

        last_version_tag = exec_command_and_return(
            "git describe --tags --abbrev=0"
        ).output

        base_default = "-Dsonar.issuesReport.console.enable=true -Dsonar.sourceEncoding=UTF-8 -Dsonar.qualitygate.wait=true"

        if self.ci_commit_branch:
            base_default = f"{base_default} -Dsonar.branch.name={self.ci_commit_branch}"

        if self.sonar_scanner_type == SonarScannerType.FLUTTER:
            base_default = (
                f"{base_default} -Dsonar.dart.analysis.reportPath=analyzer-output.txt"
            )

        base_urls = (
            f"-Dsonar.host.url={self.sonar_url} -Dsonar.links.scm={ci_project_url}"
        )
        base_ref = f"-Dsonar.projectKey={self.ci_project_path_slug} -Dsonar.projectName={self.ci_project_name}"
        base_version = f"-Dsonar.projectVersion={last_version_tag}"

        base_initial = "sonar-scanner"

        if self.sonar_scanner_type in [SonarScannerType.MAVEN, SonarScannerType.SDL]:
            base_initial = "mvn -T 1C sonar:sonar --batch-mode --errors"

            if self.sonar_scanner_type == SonarScannerType.SDL:
                properties = self._load_properties("sdl.properties")

                try:
                    output = properties["generator.java.output"]
                except KeyError:
                    output = "java"

                base_initial = (
                    f"{base_initial} --projects !client,!server --file {output}"
                )

        base_sonar_scanner = (
            f"{base_initial} {base_default} {base_urls} {base_ref} {base_version}"
        )

        return base_sonar_scanner.rstrip()

    def _config(self):
        """Check for deprecated primitives on commit"""

        mutation = """
            mutation ($gitlabId: Int!, $sonarKey: String!) {
                configSonarProject(gitlabId: $gitlabId, sonarKey: $sonarKey) {
                    success
                    result
                }
            }
        """

        params = {
            "gitlabId": int(get_env_variable("CI_PROJECT_ID")),
            "sonarKey": self.ci_project_path_slug,
        }

        self.graphql_client.call(mutation, params)

    @staticmethod
    def _load_properties(filepath, sep="=", comment_char="#"):
        """
        Read the file passed as parameter as a properties file.
        """
        props = {}
        path = Path(filepath)
        if path.is_file():
            with open(path, "rt", encoding="utf-8") as property_file:
                for line in property_file:
                    line_striped = line.strip()
                    if line_striped and not line_striped.startswith(comment_char):
                        key_value = line_striped.split(sep)
                        key = key_value[0].strip()
                        value = sep.join(key_value[1:]).strip().strip('"')
                        props[key] = value
            return props

        exit_message(
            "O projeto não é um projeto SDL, ele não possui o arquivo sdl.properties."
        )
